# flake8: noqa

"""
    Amber Electric Public API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0
    Generated by: https://openapi-generator.tech
"""


__version__ = "1.0.0"

# import Configuration
from amberelectric.configuration import Configuration

# import exceptions
from amberelectric.exceptions import OpenApiException
from amberelectric.exceptions import ApiAttributeError
from amberelectric.exceptions import ApiTypeError
from amberelectric.exceptions import ApiValueError
from amberelectric.exceptions import ApiKeyError
from amberelectric.exceptions import ApiException
